#!/bin/sh

# $Id: 9fbe9eac4407dc72d23be3b710904a17590630e1 $

s_term() {
    #EC_OK
    exit 0
}

trap s_term 15

die() {
    retval=$1
    shift
    echo "Fatal error: $*" >&2
    exit $retval
}

warning() {
    echo "Warning: $*" >&2
}

debug() {
    if [ -n "$DEBUG" ] ; then
        echo "Debug: $*" >&2
    fi
}

get_platform() {
    uname -s | LC_ALL=C tr '[A-Z]' '[a-z]'
}

get_lib_path() {
    binary="$1"
    lib="$2"
    $LDD $LDD_RECURSIVE "$binary" 2>&1 | grep "$lib" | head -n1 | while read lib dummy path ; do echo $path ; done | cut -d' ' -f1
}

binary_grep() {
    pattern="$1"
    binary="$2"
    retval=0
    [ -r "$binary" ] || return 1
    if grep --help 2>&1 | grep 'GNU' >/dev/null 2>&1 ; then
        grep -q "$pattern" "$binary" >/dev/null 2>&1
        retval=$?
    else
        #sun generic grep cannot parse binary
        cat "$binary" 2>/dev/null | sed "s/$pattern/@@@OK@@@/" 2>/dev/null | grep '@@@OK@@@' >/dev/null 2>&1
        retval=$?
    fi
    return $retval
}

append_to_library_path() {
    debug "Append \"$1\" to LD_LIBRARY_PATH"
    newpath=`echo "$LIBPATH" | sed 's#:#/'"$1"':#g;s#$#/'"$1"'#'`
    new32path=`echo "$LIB32PATH" | sed 's#:#/'"$1"':#g;s#$#/'"$1"'#'`
    if [ -n "$LD_LIBRARY_PATH" ]; then
        LD_LIBRARY_PATH="$LD_LIBRARY_PATH:$newpath"
    else
        LD_LIBRARY_PATH="$newpath"
    fi
    if [ -n "$LD_32_LIBRARY_PATH" ]; then
        LD_32_LIBRARY_PATH="$LD_32_LIBRARY_PATH:$new32path"
    else
        LD_32_LIBRARY_PATH="$new32path"
    fi
    export LD_32_LIBRARY_PATH
    export LD_LIBRARY_PATH
}

check_glibc_version() {
    glibcMajor="`$LDD --version | awk '/^ldd/ { split($NF,v,/\./); print v[1]; }'`"
    glibcMinor="`$LDD --version | awk '/^ldd/ { split($NF,v,/\./); print v[2]; }'`"
    glibcBugfix="`$LDD --version | awk '/^ldd/ { split($NF,v,/\./); print v[3]; }'`"
    if [ "$glibcMinor" = "" ]; then
       warning "Could not determine system's glibc version."
       glibcMinor=0
       return
    fi
    if [ "$glibcBugfix" = "" ]; then
       glibcBugfix=0
    fi
    minVersion=`echo $MIN_GLIBC_VERSION | awk -F. '{print $1*10000+$2*100+$3}'`
    glibcVersion=`expr $glibcMajor \* 10000 + $glibcMinor \* 100 + $glibcBugfix`

    if [ $glibcVersion -lt $minVersion ]; then
        USE_OWN_GLIBC="TRUE"
        kernel=`uname -r | awk -F. '{print $1*10000+$2*100+$3}'`
        if test $kernel -ge 20600 ; then
            KERNEL="2.6"
        elif test $kernel -ge 20400 ; then
            KERNEL="2.4"
        else
            die 125 "You need kernel 2.4 or later."
        fi
    fi
}

run() {
    libs=""
    loader=""
    if [ "$PLATFORM" = "linux" ] ; then
        check_glibc_version
        if [ -n "$USE_OWN_GLIBC" ] ; then
            loader="$LIBPATH/ldwrap/$KERNEL/ld-linux.so.2"
            append_to_library_path "ldwrap/$KERNEL"
            append_to_library_path "ldwrap/gcc"
            append_to_library_path "ldwrap/libz.so.1"
        fi
        lddOutput="`LANGUAGE=C LANG=C $LDD $LDD_RECURSIVE "$BINARY" 2>&1`"
        if echo "$lddOutput" | grep gzopen64 >/dev/null 2>&1 ; then
            append_to_library_path "ldwrap/libz.so.1"
        fi
    elif [ "$PLATFORM" = "freebsd" ] ; then
        #rpath doesn't work here
        append_to_library_path ""
        #own gcc prefer for BSD
        append_to_library_path "ldwrap/gcc"
    fi
    if binary_grep "libstdc++\.so\.6" "$BINARY" ; then
        libpath="`get_lib_path "$BINARY" "libstdc++.so.6" 2>/dev/null`"
        if binary_grep "CXXABI_$MIN_CXXABI" "$libpath" ; then
            true
        else
            append_to_library_path "ldwrap/gcc"
        fi
    fi
    lddOutput="`LANGUAGE=C LANG=C $LDD $LDD_RECURSIVE "$BINARY" 2>&1`"
    libs="`echo "$lddOutput" | grep 'not found' | awk '{print $1}'| sort | uniq`"
    for lib in $libs ; do
        append_to_library_path "ldwrap/$lib"
    done

    debug "Finally LD_LIBRARY_PATH is \"$LD_LIBRARY_PATH\""
    if [ -n "$loader" ] ; then
        exec $LOADER "$BINARY" "$@"
        retval=$?
    else
        exec "$BINARY" "$@"
        retval=$?
    fi
    return $retval
}

LDD="/usr/bin/ldd"
[ -x "$LDD" ] || die 125 "ldd not found. Fix PATH and try again."

PLATFORM=`get_platform`

case "$PLATFORM" in
    *bsd*)
        LDD_RECURSIVE="${LDD_RECURSIVE:-"-a"}"
        PREFIX="${PREFIX:-"/usr/local/libexec/drweb.com"}"
        CONFDIR="${CONFDIR:-"/usr/local/etc/opt/drweb.com"}"
        ;;
    *)
        LDD_RECURSIVE="${LDD_RECURSIVE:-"-r"}"
        PREFIX="${PREFIX:-"/opt/drweb.com"}"
        CONFDIR="${CONFDIR:-"/etc/opt/drweb.com/"}"
        ;;
esac

if [ -d "$CONFDIR/ldwrap.conf.d" ] ; then
    for f in $CONFDIR/ldwrap.conf.d/*.conf ; do
        [ -r "$f" ] && . "$f"
    done
fi

LIBPATH='/opt/drweb.com/lib64:/opt/drweb.com/lib'
LIB32PATH='/opt/drweb.com/lib'
BINARY="$PREFIX/bin/`basename "$0.real"`"

[ -x "$BINARY" ] || die 104 "$BINARY not found or not executable"

MIN_GLIBC_VERSION=2.3.4
MIN_CXXABI=${CXXABI:-1.3.7}
if [ -n "$ARGS" ] ; then
    run "$@" "$ARGS"
else
    run "$@"
fi
exit $?
